

// les sens des trains
enum {SENS_PAIR=0,SENS_IMPAIR=1}; 

/////////////////////////////////////////////////////////////////////////////

class Autorisation { // classe de base des autorisations
protected:
  boolean sens=false; // pair=false   impair=true
  boolean prise=false; // verrou

  Autorisation() {} // constructeur vide
public:
  boolean prenable(boolean s); // sens pair ou impair
  void prendre(boolean s) { sens=s; prise=true; } // sens pair ou impair
  void rendre() { prise=false; }

  virtual boolean aucunTrain() { return false; }

  void init() { sens=prise=false; } // (re)initialisation   TCO
};

//  //  // //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

boolean Autorisation::prenable(boolean s) { 
  if (sens==s) return !prise; // meme sens
  return !prise && aucunTrain(); // pas meme sens
}

/////////////////////////////////////////////////////////////////////////////
// etats possibles pour les itineraires
enum {ITINERAIRE_LIBRE,ITINERAIRE_FORME,ITINERAIRE_EN_ATTENTE}; 

class Itineraire { // classe de base des itineraires
protected:
  byte etat=ITINERAIRE_LIBRE; // l'etat de l'itineraire (libre, en attente, forme, ... )
  byte no=0; // numero de bouton itineraire   TCO

  Itineraire(byte n) { no=n; } // constructeur
public:
  boolean libre()     { return etat==ITINERAIRE_LIBRE || etat==ITINERAIRE_EN_ATTENTE; }
  boolean forme()     { return etat==ITINERAIRE_FORME; }
  boolean enAttente() { return etat==ITINERAIRE_EN_ATTENTE; }

  virtual boolean formable();
  virtual void former();
  virtual boolean deformable();
  virtual void deformer();
  virtual void tracer(boolean b) {} // TCO

  boolean formation();   // essai de formation (manuelle ou en memoire)
  boolean destruction(); // essai de destruction (manuelle)
  void detruire(); // destruction (appelle par les desactions() de zone ou annulation memorisation)
  void effacer();  // effacement d'un itineraire memorise // OPTION
  
  void init() { etat=ITINERAIRE_LIBRE; } // (re)initialisation // TCO
};

//  //  // //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //

boolean Itineraire::formation() { // essai formation 
  if (formable()) { former(); etat=ITINERAIRE_FORME; tracer(true); tco(CODE_BOUTON,COULEUR_BLANCHE,no); return true; } 
  else { etat=ITINERAIRE_EN_ATTENTE; tco(CODE_BOUTON_CLI,COULEUR_BLANCHE,no); return false; }
}

boolean Itineraire::destruction() { // essai deformation 
  if (deformable()) { deformer(); etat=ITINERAIRE_LIBRE; tracer(false); 
                      tco(CODE_BOUTON,COULEUR_GRISE,no); // TCO
                      return true; } 
  return false;
}

void Itineraire::detruire() { // destruction
  if (etat==ITINERAIRE_LIBRE) return;
  if (etat==ITINERAIRE_EN_ATTENTE) return;  
  deformer(); tracer(false); 
  etat=ITINERAIRE_LIBRE; 
  tco(CODE_BOUTON,COULEUR_GRISE,no); // TCO
  } 

void Itineraire::effacer() { // effacement de la memorisation   OPTION
  etat=ITINERAIRE_LIBRE; 
  tco(CODE_BOUTON,COULEUR_GRISE,no); // TCO
}

/////////////////////////////////////////////////////////////////////////////

