
// les signaux avec BAL   
// les semaphores sont entierement automatiques, pas de manoeuvre manuelle
// ces signaux peuvent etres utilises avec les signaux sans BAL

////////////////////////////////////////////////////////////////////////////

class SemaphoreBAL:public SignalBAL { 
protected:  
  SemaphoreBAL(int n):SignalBAL(Vl,n) {} // constructeur mini
  //SemaphoreBAL(char* nm,int t,int n):SignalBAL(nm,t+SEMAPHORE+BAL,Vl,n) {} // constructeur maxi
public:
  virtual void aubiner(); // appelle par les methodes actions de zone (retrosignalisation)
  virtual void desaubiner(); // appelle par les methodes desactions de zone (retrosignalisation)

  virtual typeFeux calculFeux(); // feux a l'ouverture
  virtual void majFeux(); // appele par le signal suivant           
};

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  

void SemaphoreBAL::aubiner() { 
  if (ferme()) return; feux=S; manoeuvreEtMajFeux();
} 

void SemaphoreBAL::desaubiner() { Signal*s; 
  if (!ferme()) return; feux=calculFeux(); manoeuvreEtMajFeux();
}

typeFeux SemaphoreBAL::calculFeux() { Signal* s; 
  if (cantonOccupe()) return S; // "canton" occupe ( precaution )
  s=suivant(); if (s==NULL) { erreur(301); return E; }
  if (s->ferme()) return A; else return Vl; 
}

void SemaphoreBAL::majFeux() { typeFeux f;           
  if (ferme()) return; 
  f=calculFeux(); if (f!=feux) { feux=f; manoeuvreEtMajFeux(); } 
}

////////////////////////////////////////////////////////////////////////////

class SemaphoreBALAvertissement:public SemaphoreBAL { // le suivant est un carre (A a l'initialisation)
protected:
  SemaphoreBALAvertissement(int n):SemaphoreBAL(n) { feux=feuxInit=A; } // constructeur mini
  //SemaphoreBALAvertissement(char* nm,int t,int n):SemaphoreBAL(nm,t+SEMAPHORE+AVERTISSEMENT+BAL,n) { feux=feuxInit=A; } // constructeur maxi
};
 
////////////////////////////////////////////////////////////////////////////

class SemaphoreBALRalentissement:public SemaphoreBAL { // le suivant est un carre RR
protected:
  SemaphoreBALRalentissement(int n):SemaphoreBAL(n) { feux=feuxInit=A; } // constructeur mini
  //SemaphoreBALRalentissement(char* nm,int t,int n):SemaphoreBAL(nm,t+SEMAPHORE+RALENTISSEMENT+BAL,n) { feux=feuxInit=A; } // constructeur maxi
public:  
  virtual typeFeux calculFeux(); // feux a l'ouverture
};

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  

typeFeux SemaphoreBALRalentissement::calculFeux() { Signal* s; // feux a l'ouverture
  if (cantonOccupe()) return S; // "canton" occupe ( precaution )
  s=suivant(); if (s==NULL) { erreur(301); return E; }
  if (s->ferme())            return A;  else
  if (s->ralentissement60()) return Rc; else
  if (s->ralentissement30()) return R; 
  else                       return Vl;  
}

////////////////////////////////////////////////////////////////////////////

class CarreBAL:public SignalBAL { 
protected:
  CarreBAL(int n):SignalBAL(C,n) { modePermanent=false; } // constructeur mini
  //CarreBAL(char* nm,int t,int n):SignalBAL(nm,t+CARRE+BAL,C,n) {} // constructeur maxi
public:  
  virtual void ouvrir(); // ouverture manuelle
  virtual void fermer(); // fermeture manuelle
  
  virtual void aubiner(); // fermeture automatique (retrosignalisation
  virtual void desaubiner(); // ouverture automatique (retrosignalisation)

  virtual typeFeux calculFeux(); // feux a l'ouverture
  virtual void majFeux(); // appele par le signal suivant
};

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  

void CarreBAL::ouvrir() { 
  if (ouvert()) return; feux=calculFeux(); manoeuvreEtMajFeux();
}

void CarreBAL::fermer() { 
  if (ferme()) return; feux=C; manoeuvreEtMajFeux();
}

void CarreBAL::aubiner() { 
  if (ferme()) return; 
  if (modePermanent) feux=S; else feux=C; manoeuvreEtMajFeux();   
}

void CarreBAL::desaubiner() { 
  if (ouvert()) return; 
  if (modePermanent) feux=calculFeux(); else return; manoeuvreEtMajFeux();   
}

typeFeux CarreBAL::calculFeux() { Signal* s; 
  if (cantonOccupe()) return S; // "canton" occupe ( precaution )
  s=suivant(); if (s==NULL) { erreur(301); return E; }
  if (s->ferme()) return A; else return Vl; 
}

void CarreBAL::majFeux() { Signal* s; typeFeux f; 
  if (ferme()) return; 
  f=calculFeux(); if (f!=feux) { feux=f; manoeuvreEtMajFeux(); } 
}
  
////////////////////////////////////////////////////////////////////////////

class CarreBALRappelRalentissement:public CarreBAL {
protected:
  CarreBALRappelRalentissement(int n):CarreBAL(n) {} // constructeur mini
  //CarreBALRappelRalentissement(char* nm,int t,int n):CarreBAL(nm,t+CARRE+RAPPEL_RALENTISSEMENT+BAL,n) {} // constructeur maxi
public:  
  virtual typeFeux calculFeux(); // feux a l'ouverture
};

//  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  //  

typeFeux CarreBALRappelRalentissement::calculFeux() { Signal* s; 
  s=suivant(); if (s==NULL) { erreur(301); return E; }
  if (ralentissement60()) if (s->ferme()) return A+RRc; else return RRc; else
  if (ralentissement30()) if (s->ferme()) return A+RR;  else return RR;  
  else                    if (s->ferme()) return A;     else return Vl;
}
  
////////////////////////////////////////////////////////////////////////////

